import typing
import typing_extensions
import pydantic

from .authentication import Authentication
from .card_type_indicators import CardTypeIndicators
from .expiry import Expiry
from .network_response import NetworkResponse
from .payment_token import PaymentToken
from .payment_authentication_request import PaymentAuthenticationRequest


class VerificationCard(pydantic.BaseModel):
    """
    Card payment instrument for card verification
    """

    model_config = pydantic.ConfigDict(
        arbitrary_types_allowed=True,
        populate_by_name=True,
    )

    account_number: typing.Optional[str] = pydantic.Field(
        alias="accountNumber", default=None
    )
    """
    The card or token number.
    """
    account_number_type: typing.Optional[
        typing_extensions.Literal[
            "DEVICE_TOKEN",
            "NETWORK_TOKEN",
            "PAN",
            "SAFETECH_PAGE_ENCRYPTION",
            "SAFETECH_TOKEN",
        ]
    ] = pydantic.Field(alias="accountNumberType", default=None)
    """
    Specifies the type of payment method used by the account number in payment transaction.
    """
    authentication: typing.Optional[Authentication] = pydantic.Field(
        alias="authentication", default=None
    )
    """
    The authentication object allows you to opt in to additional security features like 3-D Secure
    """
    card_type: typing.Optional[
        typing_extensions.Literal[
            "AP",
            "AX",
            "CC",
            "CR",
            "CZ",
            "DC",
            "DI",
            "EP",
            "IM",
            "JC",
            "MC",
            "MR",
            "NP",
            "PP",
            "SP",
            "VI",
            "VR",
        ]
    ] = pydantic.Field(alias="cardType", default=None)
    """
    Abbreviation of card name
    """
    card_type_indicators: typing.Optional[CardTypeIndicators] = pydantic.Field(
        alias="cardTypeIndicators", default=None
    )
    """
    The card type indicators provide additional information about the card. For example, if the card is a prepaid card and thus less likely to         support recurring payments or if the card is a healthcare or commercial  card.
    """
    card_type_name: typing.Optional[
        typing_extensions.Literal[
            "ACCEL_PINLESS",
            "AMERICAN_EXPRESS",
            "CHASENET_CREDIT",
            "CHASENET_SIGNATURE_DEBIT",
            "CHINA_UNION_PAY",
            "DINERS_CLUB",
            "DISCOVER",
            "EFTPOS_PINLESS",
            "INTERNATIONAL_MAESTRO",
            "JCB",
            "MASTERCARD",
            "MASTERCARD_RESTRICTED_DEBIT",
            "NYCE_PINLESS",
            "PULSE_PINLESS",
            "STAR_PINLESS",
            "VISA",
            "VISA_RESTRICTED_DEBIT",
        ]
    ] = pydantic.Field(alias="cardTypeName", default=None)
    """
    Name of the payment network.
    """
    cvv: typing.Optional[str] = pydantic.Field(alias="cvv", default=None)
    """
    Card verification value (CVV/CV2)
    """
    encryption_integrity_check: typing.Optional[str] = pydantic.Field(
        alias="encryptionIntegrityCheck", default=None
    )
    """
    The alphanumeric string generated by voltage to verify the soundness of the encrypted key used by merchant and payment process. The merchant passed this in the API call. The backend process validates the subscriber id and format matches - between the merchant request for a key and the UPG request
    """
    expiry: typing.Optional[Expiry] = pydantic.Field(alias="expiry", default=None)
    """
    Expiration date
    """
    is_bill_payment: typing.Optional[bool] = pydantic.Field(
        alias="isBillPayment", default=None
    )
    """
    Indicates whether or not the transaction is identified as a bill payment, prearranged between the cardholder and the merchant.
    """
    masked_account_number: typing.Optional[str] = pydantic.Field(
        alias="maskedAccountNumber", default=None
    )
    """
    Identifies a concealed number associated with the card number recognized by various payment systems. This is typically concealed by storing only the first 6 and/or last 4 digits of the payment account number or some variation.
    """
    merchant_sales_channel_name: typing.Optional[
        typing_extensions.Literal[
            "INTERACTIVE_VOICE_RESPONSE", "INTERNET", "MAIL_ORDER_TELEPHONE_ORDER"
        ]
    ] = pydantic.Field(alias="merchantSalesChannelName", default=None)
    """
    Label given to a merchant client of the Firm's medium for reaching its customers and facilitating and/or performing sales of its merchandise or services.
    """
    network_response: typing.Optional[NetworkResponse] = pydantic.Field(
        alias="networkResponse", default=None
    )
    """
    Response information from payment network
    """
    original_network_transaction_id: typing.Optional[str] = pydantic.Field(
        alias="originalNetworkTransactionId", default=None
    )
    """
    When submitting a merchant-initiated payment, submit the networkTransactionId received from the first payment in this field.
    """
    payment_tokens: typing.Optional[typing.List[PaymentToken]] = pydantic.Field(
        alias="paymentTokens", default=None
    )
    """
    List of payment tokens for the transaction
    """
    token_service_response_code: typing.Optional[str] = pydantic.Field(
        alias="tokenServiceResponseCode", default=None
    )
    """
    Short explanation of response Code
    """
    verification_authentication_request: typing.Optional[
        PaymentAuthenticationRequest
    ] = pydantic.Field(alias="verificationAuthenticationRequest", default=None)
    """
    Request Authentication during payment process
    """
    wallet_provider: typing.Optional[
        typing_extensions.Literal["APPLE_PAY", "GOOGLE_PAY"]
    ] = pydantic.Field(alias="walletProvider", default=None)
    """
    Identifies the organization who manages the e-wallet for a consumer. The actual e-wallet management responsibilities may include hosting, accessing, communicating, and/or updating all or some of the customer data associated with the e-wallet.  An E-wallet is an application that is created on the mobile device to interact with the Point of Sale (POS) device as a catalyst for a transaction. This value may be sent to the Firm (as in Visa Tokenization) or created by the Firm.
    """
