import typing
import typing_extensions
import pydantic

from .account_holder_information import (
    AccountHolderInformation,
    _SerializerAccountHolderInformation,
)
from .fraud_score import FraudScore, _SerializerFraudScore
from .merchant import Merchant, _SerializerMerchant
from .fraud_check_payment_method_type import (
    FraudCheckPaymentMethodType,
    _SerializerFraudCheckPaymentMethodType,
)
from .fraud_ship_to import FraudShipTo, _SerializerFraudShipTo


class FraudCheckRequest(typing_extensions.TypedDict):
    """
    Object for passing fraud check request parameters
    """

    account_holder: typing_extensions.NotRequired[AccountHolderInformation]
    """
    Information about the card Account Holder for which fraud checking is performed.
    """

    amount: typing_extensions.Required[int]
    """
    Total monetary value of the payment including all taxes and fees.
    """

    currency: typing_extensions.Required[
        typing_extensions.Literal[
            "AED",
            "AFN",
            "ALL",
            "AMD",
            "ANG",
            "AOA",
            "ARS",
            "AUD",
            "AWG",
            "AZN",
            "BAM",
            "BBD",
            "BDT",
            "BGN",
            "BIF",
            "BMD",
            "BND",
            "BOB",
            "BRL",
            "BSD",
            "BTN",
            "BWP",
            "BYN",
            "BZD",
            "CAD",
            "CDF",
            "CHF",
            "CLP",
            "CNY",
            "COP",
            "CRC",
            "CVE",
            "CZK",
            "DJF",
            "DKK",
            "DOP",
            "DZD",
            "EGP",
            "ETB",
            "EUR",
            "FJD",
            "FKP",
            "GBP",
            "GEL",
            "GHS",
            "GIP",
            "GMD",
            "GTQ",
            "GYD",
            "HKD",
            "HNL",
            "HRK",
            "HTG",
            "HUF",
            "IDR",
            "ILS",
            "INR",
            "ISK",
            "JMD",
            "JPY",
            "KES",
            "KHR",
            "KMF",
            "KRW",
            "KYD",
            "KZT",
            "LAK",
            "LBP",
            "LKR",
            "LRD",
            "LSL",
            "MAD",
            "MDL",
            "MGA",
            "MKD",
            "MMK",
            "MNT",
            "MOP",
            "MRU",
            "MUR",
            "MVR",
            "MWK",
            "MXN",
            "MYR",
            "MZN",
            "NAD",
            "NGN",
            "NIO",
            "NOK",
            "NPR",
            "NZD",
            "PAB",
            "PEN",
            "PGK",
            "PHP",
            "PKR",
            "PLN",
            "PYG",
            "QAR",
            "RON",
            "RSD",
            "RWF",
            "SAR",
            "SBD",
            "SCR",
            "SEK",
            "SGD",
            "SHP",
            "SLL",
            "SOS",
            "SRD",
            "STN",
            "SZL",
            "THB",
            "TJS",
            "TOP",
            "TRY",
            "TTD",
            "TWD",
            "TZS",
            "UAH",
            "UGX",
            "USD",
            "UYU",
            "UZS",
            "VND",
            "VUV",
            "WST",
            "XAF",
            "XCD",
            "XOF",
            "XPF",
            "YER",
            "ZAR",
            "ZMW",
        ]
    ]
    """
    Describes the currency type of the transaction
    """

    fraud_score: typing_extensions.NotRequired[FraudScore]
    """
    Object for Fraud Score Information
    """

    merchant: typing_extensions.NotRequired[Merchant]
    """
    Information about the merchant
    """

    payment_method_type: typing_extensions.Required[FraudCheckPaymentMethodType]
    """
    Object with information for Payment Method Type for  Fraud Check
    """

    ship_to: typing_extensions.NotRequired[FraudShipTo]
    """
    Ship To Information used for fraud checking services.
    """


class _SerializerFraudCheckRequest(pydantic.BaseModel):
    """
    Serializer for FraudCheckRequest handling case conversions
    and file omissions as dictated by the API
    """

    model_config = pydantic.ConfigDict(
        populate_by_name=True,
    )

    account_holder: typing.Optional[_SerializerAccountHolderInformation] = (
        pydantic.Field(alias="accountHolder", default=None)
    )
    amount: int = pydantic.Field(
        alias="amount",
    )
    currency: typing_extensions.Literal[
        "AED",
        "AFN",
        "ALL",
        "AMD",
        "ANG",
        "AOA",
        "ARS",
        "AUD",
        "AWG",
        "AZN",
        "BAM",
        "BBD",
        "BDT",
        "BGN",
        "BIF",
        "BMD",
        "BND",
        "BOB",
        "BRL",
        "BSD",
        "BTN",
        "BWP",
        "BYN",
        "BZD",
        "CAD",
        "CDF",
        "CHF",
        "CLP",
        "CNY",
        "COP",
        "CRC",
        "CVE",
        "CZK",
        "DJF",
        "DKK",
        "DOP",
        "DZD",
        "EGP",
        "ETB",
        "EUR",
        "FJD",
        "FKP",
        "GBP",
        "GEL",
        "GHS",
        "GIP",
        "GMD",
        "GTQ",
        "GYD",
        "HKD",
        "HNL",
        "HRK",
        "HTG",
        "HUF",
        "IDR",
        "ILS",
        "INR",
        "ISK",
        "JMD",
        "JPY",
        "KES",
        "KHR",
        "KMF",
        "KRW",
        "KYD",
        "KZT",
        "LAK",
        "LBP",
        "LKR",
        "LRD",
        "LSL",
        "MAD",
        "MDL",
        "MGA",
        "MKD",
        "MMK",
        "MNT",
        "MOP",
        "MRU",
        "MUR",
        "MVR",
        "MWK",
        "MXN",
        "MYR",
        "MZN",
        "NAD",
        "NGN",
        "NIO",
        "NOK",
        "NPR",
        "NZD",
        "PAB",
        "PEN",
        "PGK",
        "PHP",
        "PKR",
        "PLN",
        "PYG",
        "QAR",
        "RON",
        "RSD",
        "RWF",
        "SAR",
        "SBD",
        "SCR",
        "SEK",
        "SGD",
        "SHP",
        "SLL",
        "SOS",
        "SRD",
        "STN",
        "SZL",
        "THB",
        "TJS",
        "TOP",
        "TRY",
        "TTD",
        "TWD",
        "TZS",
        "UAH",
        "UGX",
        "USD",
        "UYU",
        "UZS",
        "VND",
        "VUV",
        "WST",
        "XAF",
        "XCD",
        "XOF",
        "XPF",
        "YER",
        "ZAR",
        "ZMW",
    ] = pydantic.Field(
        alias="currency",
    )
    fraud_score: typing.Optional[_SerializerFraudScore] = pydantic.Field(
        alias="fraudScore", default=None
    )
    merchant: typing.Optional[_SerializerMerchant] = pydantic.Field(
        alias="merchant", default=None
    )
    payment_method_type: _SerializerFraudCheckPaymentMethodType = pydantic.Field(
        alias="paymentMethodType",
    )
    ship_to: typing.Optional[_SerializerFraudShipTo] = pydantic.Field(
        alias="shipTo", default=None
    )
